					MEMBER
					MAP
					END

! Generic string handling class method declarations.
                    
	INCLUDE('System_String.INC'),ONCE
	include('System_Diagnostics_Debugger.inc'),once

  !Public Methods
!!!  
!!! <summary>Assign a new value to the string class</summary>
!!! <param name="NewValue">The new string to assign to the class</param>
!!! <remarks>A new value can be assigned to the class regardless
!!! if it already has a value. The old value is automatically disposed.</remarks>
System_String.Assign        PROCEDURE(STRING pNewValue)
strLen                          LONG,AUTO
	CODE
	Self.DisposeStr()
	strLen = LEN(pNewValue)
	IF strLen > 0
		Self.Value &= NEW STRING(strLen)
		Self.Value = pNewValue
	END
    
!!!
!!! <summary>Append the new value to the end of the existing class string.</summary>
!!! <remarks>If no value already exists then the new value is assigned
!!! as if Assign had been called instead of Append.</remarks>
System_String.Append        PROCEDURE(STRING pNewValue)
	CODE
	IF NOT Self.Value &= NULL
		Self.Assign(Self.Value & pNewValue)
	ELSE
		Self.Assign(pNewValue)
	END

System_String.BeginsWith    procedure(string s)!,byte
thislength                      long
otherlength                     long
	CODE
	s = upper(clip(s))
	IF NOT Self.Value &= NULL
		if s = '' 
			return FALSE
		end
		thislength = len(clip(self.value))
		otherlength = len(s)
		if otherlength > thislength
			return FALSE
		end
		if SUB(upper(self.value),1,otherlength) = s
			return TRUE
		end			
	END
	return false
	
	

!!!
!!! <summary>Return the offset of the sub-string in existing class string.</summary>
!!! <param name="TestValue">Sub-string to search for</param>
!!! <param name="NoCase">Optional parameter to ignore case. Default is case-sensitive.</param>
!!! <remarks>If the sub-string does not exist then zero is returned.</remarks>
System_String.Contains      PROCEDURE(STRING pTestValue, LONG pNoCase=0)!,LONG
	CODE
	RETURN InString(Choose(pNoCase,UPPER(pTestValue),pTestValue),Choose(pNoCase,UPPER(Self.Value),Self.Value),1,1)

!!!
!!! <summary>Append the new value to the beginning of the existing class string.</summary>
!!! <remarks>If no value already exists then the new value is assigned
!!! as if Assign had been called instead of Append.</remarks>
System_String.Prepend       PROCEDURE(STRING pNewValue)
	CODE
	IF NOT Self.Value &= NULL
		Self.Assign(pNewValue & Self.Value)
	ELSE
		Self.Assign(pNewValue)
	END

!!!
!!! <summary>Deprecated - use Prepend method.</summary>
System_String.PreAppend     PROCEDURE(STRING pNewValue)
	CODE
	self.Prepend(pNewvalue)
!!!
!!! <summary>Deallocate dynamic memory when class goes out of scope.</summary>
System_String.Destruct      PROCEDURE()
	CODE
	Self.DisposeStr()
	Self.DisposeLines()

System_String.EndsWith      procedure(string s)!,byte
thislength                      long
otherlength                     long
	CODE
	s = upper(clip(s))
	IF NOT Self.Value &= NULL
		if s = ''
			return FALSE
		end
		thislength = len(clip(self.value))
		otherlength = len(s)
		if otherlength > thislength
			return FALSE
		end
		if SUB(upper(self.value),thislength-otherlength+1,otherlength) = s
			return TRUE
		end			
	END
	return false

!!!
!!! <summary>Return current string</summary>
!!! <remarks>If no string has been assigned an empty string is returned.</remarks>
System_String.Get   PROCEDURE() !,STRING
	CODE
	IF NOT Self.Value &= NULL
		RETURN Self.Value
	ELSE
		RETURN ''
	END

!!!
!!! <summary>Return the index of the start of the specified string value.</summary>
!!! <remarks>If no match is found zero is returned.</remarks>
System_String.IndexOf       PROCEDURE(string s) !,LONG
	CODE
	IF NOT Self.Value &= NULL
		return INSTRING(s,self.value,1)
	END
	return 0;

!!!
!!! <summary>Return the last index of the start of the specified string value.</summary>
!!! <remarks>If no match is found zero is returned.</remarks>
System_String.LastIndexOf   PROCEDURE(string s) !,LONG
idx                             long
lastidx                         long(0)
startidx                        long(1)
msg                             cstring(100)
	CODE
	LOOP
		idx = instring(s,self.value,1,startidx)
		if idx = 0 then break end
		lastidx = idx
		startidx = idx + 1
	END
	return lastidx

!!!
!!! <summary>Return the last index of the start of the specified string value.</summary>
!!! <remarks>If no match is found zero is returned.</remarks>
System_String.InsertAt      PROCEDURE(string s, long pos) !,LONG
insertPosition                  long
	CODE
	if pos < 1 then pos = 1 end
	IF NOT Self.Value &= NULL
		insertPosition = pos 
		if self.length() < pos 
			self.Append(s)
		ELSE
			!self.value = self.substring(1,pos) & s & self.substring(pos+1,self.length())
			!self.value = self.substring(1,pos-1) & s & self.substring(pos,self.length())
			self.assign(self.substring(1,pos-1) & s & self.substring(pos,self.length()))
		end
	END
	

!!!
!!! <summary>Return the length of the existing string value.</summary>
!!! <remarks>If no string has been assigned zero is returned.</remarks>
System_String.Length        PROCEDURE() !,LONG
	CODE
	IF NOT Self.Value &= NULL
		RETURN LEN(Self.Value)
	ELSE
		RETURN 0
	END

!!!
!!! <summary>Replace occurences of one string with another in class value.</summary>
!!! <param name="OldValue">Sub-string to search for</param>
!!! <param name="NewValue">New value to replace with</param>
!!! <param name="Count">Optional parameter: How many occurences to replace. Default is all.</param>
!!! <remarks>This operation is non-overlapping. If the OldValue occurs in the NewValue the
!!! occurences from inserting NewValue will not be replaced.</remarks>
System_String.Replace       PROCEDURE(STRING pOldValue, STRING pNewValue,<LONG pCount>)
lCount                          LONG,AUTO
lStrPos                         LONG,AUTO
lStartPos                       LONG(1)
dbg                             System_Diagnostics_Debugger

	CODE
	!dbg.SetPrefix('System_String.Replace')
	!dbg.write('Current string: ' & self.value)
	!dbg.write('replacing ' & pOldValue & ' with ' & pNewValue)
	IF NOT Self.Value &= NULL
		LOOP
			lStrPos = INSTRING(pOldValue,Self.Value,1,lStartPos)
			!dbg.write('lstrPos ' & lstrpos)
			IF lStrPos
				Self.Assign(Self.Value[1 : lStrPos-1 ] & pNewValue & Self.Value[ (lStrPos + LEN(pOldValue)) : LEN(Self.Value) ])
				!dbg.write('New string: ' & self.value)				
				lStartPos = lStrPos + LEN(pNewValue)
				lCount += 1
				IF NOT OMITTED(pCount) AND lCount = pCount
					BREAK
				END
			ELSE
				BREAK
			END
		END
	END

!!!
!!! <summary>Count the occurences of a sub-string in class value.</summary>
!!! <param name="SearchValue">Sub-string to search for</param>
!!! <param name="StartPos">Optional parameter to indicate what position to start search. Default is beginning.</param>
!!! <param name="EndPos">Optional parameter to indicate what position to end search. Default is end of string.</param>
!!! <param name="NoCase">Optional parameter: Ignore case in comparision. Default is case-sensitive.</param>
System_String.Count PROCEDURE(STRING pSearchValue, <LONG pStartPos>, <LONG pEndPos>, BYTE pNoCase=0) !,LONG
lCount                  LONG(0)
lStrPos                 LONG,AUTO
lStartPos               LONG(1)
SearchString            System_String
	CODE
	IF NOT Self.Value &= NULL
		IF OMITTED(pStartPos) AND OMITTED(pEndPos)
			SearchString.Assign(Self.Value)
		ELSIF OMITTED(pStartPos) 
			SearchString.Assign(Self.SubString(1,pEndPos))
		ELSE
			SearchString.Assign(Self.SubString(pStartPos,Self.Length()))
		END
		IF pNoCase
			SearchString.Assign(UPPER(SearchString.Get()))
			pSearchValue = UPPER(pSearchValue)
		END
		LOOP
			lStrPos = INSTRING(pSearchValue,SearchString.Get(),1,lStartPos)
			IF lStrPos
				lStartPos = lStrPos + LEN(pSearchValue)
				lCount += 1
			ELSE
				BREAK
			END
		END
	END
	RETURN lCount

!!!
!!! <summary>Return specific line after calling Split method.</summary>
!!! <param name="LineNumber">Line to return. If LineNumber is greater than the number of lines in queue
!!! then an empty string is returned.</param>
!!! <remarks>If split has not been called an empty string is returned.</remarks>
System_String.GetLine       PROCEDURE(LONG pLineNumber) !,STRING
	CODE
	IF Self.Lines &= NULL
		RETURN ''
	ELSE
		GET(Self.Lines,pLineNumber)
		IF ERRORCODE()
			RETURN ''
		ELSE
			RETURN Self.Lines.Line
		END
	END
    
!!!
!!! <summary>Return the number of lines a string value was broken into after calling Split.</summary>
!!! <remarks>If split has not been called zero is returned.</remarks>
System_String.Records       PROCEDURE() !,LONG
	CODE
	IF Self.Lines &= NULL
		RETURN 0
	ELSE
		RETURN RECORDS(Self.Lines)
	END

!!!
!!! <summary>Breakdown the current string value into a series of string. Use the passed string value
!!! as a delimiter.</summary>
!!! <param name="SplitStr">Sub-String used to break up string. </param>
!!! <remarks>The sub-string is consumed by the command and does not appear in the lines.
!!! Use Records and GetLine methods to return information about the split queue.</remarks>
System_String.Split PROCEDURE(STRING pSplitStr)
q                       queue
s                           string(100)
						end
	CODE
	q.s = pSplitStr
	add(q)
	self.Split(q)
	

!!!
!!! <summary>Breakdown the current string value into a series of string. Use the queue of passed string values
!!! as delimiters.</summary>
!!! <param name="SplitStrQ">Single-value queue of substrings used to break up string. </param>
!!! <remarks>The substrings are consumed by the command and do not appear in the lines.
!!! Use Records and GetLine methods to return information about the split queue.</remarks>
System_String.Split PROCEDURE(queue q)
SplitStrPos             LONG,AUTO
LowestSplitStrPos       LONG,AUTO
StartPos                LONG(1)
x                       long
y                       long
z                       long
DelimiterQ              QUEUE
value                       cstring(100)
						end
chars                   long
	CODE
	loop x = 1 to records(q)
		get(q,x)
		DelimiterQ.Value = clip(q)
		if len(DelimiterQ.Value) = 0
			delimiterQ.value = ' '
		end
		add(DelimiterQ)
	end
	IF NOT Self.Value &= NULL
		Self.DisposeLines
		Self.Lines &= NEW(SplitStringQType)
		!gdbg.write('splitting ' & self.value)
		LOOP
			LowestSplitStrPos = 0
			SplitStrPos = 0
			loop x = 1 to records(DelimiterQ)
				get(DelimiterQ,x)
				!gdbg.write('testing delimiter >' & delimiterq.value & '<<')
				!SplitStrPos = INSTRING(DelimiterQ.Value,Self.Value,1,StartPos)
				chars = len(delimiterq.value) - 1
				loop y = StartPos to self.length()
					z  = y + chars
					if self.value[y : y+chars] = delimiterq.value
						if y < LowestSplitStrPos or LowestSplitStrPos = 0
							LowestSplitStrPos = y
						END
					end
				end
			end
			SplitStrPos = LowestSplitStrPos
			!gdbg.Write('SplitStrPos: ' & SplitStrPos)
			IF SplitStrPos
				Self.Lines.Line &= NEW(STRING(LEN(Self.Value[StartPos : SplitStrPos-1])))
				Self.Lines.Line = Self.Value[StartPos : SplitStrPos-1]
				!gdbg.write('Line: >' & self.lines.line & '<<')
				if self.Lines.Line <> ''
					ADD(Self.Lines)
				end
				StartPos = SplitStrPos + LEN(DelimiterQ.Value)
				IF StartPos > LEN(Self.Value)
					BREAK
				END
				IF STARTPOS + 100 > LEN(Self.Value)
					Z# =  1 !Debug
				END
			ELSE
				Self.Lines.Line &= NEW(STRING(LEN(Self.Value[StartPos : LEN(Self.Value)])))
				Self.Lines.Line = Self.Value[StartPos : LEN(Self.Value)]
				ADD(Self.Lines)
				BREAK
			END
		END
	END

!!!
!!! <summary>Return sub-string from the current string value.</summary>
!!! <param name="Start">Start of sub-string.</param>
!!! <param name="Stop">Stop position of sub-string.</param>
!!! <remarks>If the Stop position is greater than the length of the string the string length is used
!!! as the stop position. If the Start position greater than the stop position or the length
!!! of the string then an empty string is returned.</remarks>
System_String.SubString     PROCEDURE(LONG pStart, LONG pStop) !,STRING
	CODE
	IF pStop > LEN(SELF.Value)
		pSTOP = LEN(SELF.VALUE)
	ELSIF pStart > LEN(Self.Value)
		RETURN ''
	ELSIF pStart > pStop
		RETURN ''
	END
	RETURN SELF.Value[pStart : pStop]  

  !Private Methods
!!!
!!! <summary>Private method to dispose of dynamic memory allocated by Split method.</summary>
System_String.DisposeLines  PROCEDURE() !,PRIVATE
I                               LONG
	CODE
	IF NOT Self.Lines &= NULL
		LOOP I = 1 TO RECORDS(Self.Lines)
			GET(Self.Lines,I)
			DISPOSE(Self.Lines)
		END
		FREE(Self.Lines)
	END

!!!
!!! <summary>Private method to dispose of string value.</summary>
System_String.DisposeStr    PROCEDURE() !,PRIVATE
	CODE
	IF NOT Self.Value &= NULL
		DISPOSE(Self.Value)
	END
